import unicodedata
import curses
from collections.abc import Callable
from schedule import Job
import schedule
# 算出有几个非英文字符


def wide_chars(s):
    return sum(unicodedata.east_asian_width(x) == "W" for x in s)


# 非英文字符都算2个长度, 加上去
def width(s) -> int:
    return len(s) + wide_chars(s)


# 获取最长
def get_max_lens(rows: list):
    # 初始化为 0
    max_len = [0] * len(rows[0])
    for row in rows:
        for i in range(len(row)):
            # 要转 str, 避免错误
            str_width = width(str(row[i]))
            # print(row[i])
            # print(str_width)
            if str_width > max_len[i]:
                max_len[i] = str_width
    return max_len


# 格式化并修复 ljust bug
def format_rows(rows: list, spliter: str) -> list:
    max_lens = get_max_lens(rows)
    formated_rows = []
    for row in rows:
        # formated_row = [str(row[i]).ljust(max_lens[i]) for i in range(len(row))]
        # ljust 使用 len 判断长度, 支持非英文字符, 导致对每个非英文字符多填充了一个空白
        # 解决办法是算出有n个非英字符, just 长度-n
        formated_row = [
            str(row[i]).ljust(max_lens[i] - wide_chars(str(row[i])))
            for i in range(len(row))
        ]
        formated_rows.append(spliter.join(formated_row))
    return formated_rows


def table(rows: list, spliter: str):
    for i in format_rows(rows, spliter):
        print(i)


# 刷新绘制, 不会重复输出表格内容
def curses_table(get_rows: Callable, job: Job, spliter: str = "    "):
    """
    get_rows 返回值为 rows 的函数
    job 定义的刷新周期, 比如 schedule.every(1).minutes
    """
    stdscr = curses.initscr()

    def do():
        stdscr.clear()
        rows = get_rows()
        for i in format_rows(rows, spliter):
            stdscr.addstr(i + "\n")
        stdscr.refresh()
        # stdscr.getkey()

    job.do(do)
    do()
    while True:
        schedule.run_pending()


if __name__ == "__main__":
    import time

    def get_rows():
        return [
            ["用户名", "服务器地址", "服务器的一些说明"],
            ["用户名User", f"{time.time()}", "Description"],
            ["User", "Host", "Description"],
            ["root", "h.bigzhu.net", "dump"],
            ["root", "racknerd.bigzhu.net", "racknerd"],
            ["bigzhu", "ssh.entube.app", "digitalocean"],
        ]

    # table(get_rows(), " ")
    curses_table(get_rows, schedule.every(4).seconds)



"""
wrap telegram bot
need put .env file in the same folder and set TEL_TOKEN CHAT_ID
"""
from .send_photo import send_photo
from .send_message import send_message

__version__ = "0.1.0"
__author__ = 'Big Zhu'



from .table import table


def test(capfd):
    data = [
        ["User", "Host", "Description"],
        ["root", "h.bigzhu.net", "dump"],
        ["root", "racknerd.bigzhu.net", "racknerd"],
        ["bigzhu", "ssh.entube.app", "digitalocean"],
    ]

    table(data, "~")
    out, err = capfd.readouterr()
    # print(repr(out))
    assert out == '''User  ~Host               ~Description \nroot  ~h.bigzhu.net       ~dump        \nroot  ~racknerd.bigzhu.net~racknerd    \nbigzhu~ssh.entube.app     ~digitalocean\n'''
    assert err == ''



from datetime import datetime
import pytz

# Get the timezone object for China
tz_SH = pytz.timezone("Asia/Shanghai")


def cn_now() -> str:
    datetime_SH = datetime.now(tz_SH)
    return datetime_SH.strftime("%Y-%m-%d %H:%M:%S")


if __name__ == "__main__":
    print(cn_now())



#!/usr/bin/env python3
import json
import os
import time
from zbig.zhash.args import args_hash
# 使用 json 缓存函数


# 默认 4 小时: 14400 秒
def cache(life_second=14400):
    # check is file modify time in 4 hours
    def check_file_is_alive(file_name: str) -> bool:
        file = f"{file_name}.json"
        if not os.path.exists(file):
            # print("file not exist")
            return False
        # get the modify time of the file
        modify_time = os.path.getmtime(file)
        # get the current time
        current_time = time.time()
        # check if the file modify time is out of 4 hours ago
        return current_time - modify_time <= life_second

    def read_from_file(file_name: str):
        with open(f"{file_name}.json", "r") as outfile:
            data = json.load(outfile)
        return data

    def save_to_file(file_name: str, content):
        with open(f"{file_name}.json", "w") as outfile:
            json.dump(content, outfile)

    def decorator(fn):
        def wrapped(*args, **kwargs):
            # 用函数名和入参作为 key
            hash_name = args_hash(*args, **kwargs)
            global file_name
            file_name = f"{fn.__name__}_{hash_name}"
            if check_file_is_alive(file_name):
                res = read_from_file(file_name)
                if res:
                    # print("get data from file")
                    return res
            res = fn(*args, **kwargs)
            save_to_file(file_name, res)
            return res

        return wrapped

    return decorator


@cache()
def get_token(name: str):
    # imaginary API call to get token
    return {"name": name, "time": time.time()}


if __name__ == "__main__":
    print(get_token("bigzhu"))
    print(get_token("bigzhu2"))



from .define import bot, CHAT_ID


def send_photo(file_path: str, caption: str):
    photo = open(file_path, 'rb')
    bot.send_photo(chat_id=CHAT_ID, photo=photo, caption=caption)


if __name__ == "__main__":
    send_photo('/home/bigzhu/Pictures/00006.jpeg', 'test')



from zbig.ztime import cn_now
from .define import bot, CHAT_ID


def send_message(message: str):
    bot.send_message(chat_id=CHAT_ID, text=f"{cn_now()} {message}")


if __name__ == "__main__":
    send_message('zbig send_message test')



from .csv import read_csv


def test_read_csv():
    header, rows = read_csv("hosts.csv")
    assert header == ['User', 'Host', 'Description']
    assert rows == [['bigzhu', 'ssh.entube.app', 'digitalocean'], ['root', 'google.com', 'just test']]
